/***************************************************************************
 *   Copyright (C) 2009 by Herbert Roider   *
 *   herbert.roider@utanet.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *

 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef UTILITY_H
#define UTILITY_H

#include <math.h>
#include <fstream>

/**unabhängige funktionen die für ballistic gebraucht werden, und auch sonst verwendet werden können
 *@author Herbert Roider
 */

#define RAD_TO_GRAD(rad) (rad*565.48667764616278292328)
#define GRAD_TO_RAD(grad) (grad*0.01745329251994329577)

namespace arw {
	namespace utility {

		class vector_3d {

		public:
			double x, y, z;

			vector_3d & operator *=(double s) {
				x *= s;
				y *= s;
				z *= s;
				return *this;
			}
			vector_3d & operator /=(double s) {
				x /= s;
				y /= s;
				z /= s;
				return *this;
			}
			vector_3d & operator +=(double s) {
				x += s;
				y += s;
				z += s;
				return *this;
			}
			vector_3d & operator -=(double s) {
				x -= s;
				y -= s;
				z -= s;
				return *this;
			}
			vector_3d & operator +=(const vector_3d &s) {
				x += s.x;
				y += s.y;
				z += s.z;
				return *this;
			}
			vector_3d & operator -=(const vector_3d &s) {
				x -= s.x;
				y -= s.y;
				z -= s.z;
				return *this;
			}
			vector_3d & operator *=(const vector_3d &s) {
				x *= s.x;
				y *= s.y;
				z *= s.z;
				return *this;
			}
			vector_3d & operator /=(const vector_3d &s) {
				x /= s.x;
				y /= s.y;
				z /= s.z;
				return *this;
			}
			bool operator ==(const vector_3d &s) {
				if(x != s.x)
					return false;
				if(y != s.y)
					return false;
				if(z != s.z)
					return false;
				return true;
			}

		};

		vector_3d operator+(const vector_3d &v1, const vector_3d &v2);
		vector_3d operator-(const vector_3d &v1, const vector_3d &v2);
		vector_3d operator*(const vector_3d &v1, const vector_3d &v2);
		vector_3d operator/(const vector_3d &v1, const vector_3d &v2);

		vector_3d operator+(const vector_3d &v1, double d);
		vector_3d operator-(const vector_3d &v1, double d);
		vector_3d operator*(const vector_3d &v1, double d);
		vector_3d operator/(const vector_3d &v1, double d);




		std::ostream & operator<<(std::ostream &os, const vector_3d &v);

		const double HALF_OF_PI = 3.1415926535897932384626433832795 / 2.0;//90 degree to rad
		const double PI = 3.1415926535897932384626433832795;

		//speed of sound by mach=1 in m/s at Sea level @see http://en.wikipedia.org/wiki/Mach_number#Overview
		const double g_MACH1 = 340.3;

		const double g_max_flight_time = 200.0; //200 sec.
		const double g_AIR_DENSITY = 1.2041;//kg/m³

		const double g_G = 9.81; //Erdbeschleunigung


		/* lineare  interpolation: */
		double linint(double x0, double y0, double x1, double y1, double x);

		void matrix_rotate(double m[4][4], double r[3], double angle);
		void point_transform(vector_3d & p, double m[4][4]);
		void matrix_zero(double m[4][4]);

		void matrix_identity(double m[4][4]);
		void matrix_product(double m[4][4], double m1[4][4], double m2[4][4]);

		void matrix_translate(double m[4][4], double t[3]);

		void rotate(vector_3d & point, double axis[3], double angle);
		void rotate(vector_3d & point, vector_3d axis, double angle);
		void rotate(vector_3d & point, vector_3d axis);
		void translate(vector_3d & point, vector_3d t);
		void vector_normalize(vector_3d & v);
		void vector_normalize(double v[3]);
		vector_3d normal_vector(const vector_3d &p, const vector_3d &q);
		vector_3d rotate2xAxis(const vector_3d &v1);
		double angle(const vector_3d &p, const vector_3d &q);

	};
};

#endif
